#define _GNU_SOURCE

#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <time.h>
#include <arpa/inet.h>
#include <stdarg.h>

#include <Conn.h>

/* Global variables */
static unsigned short	debug = 20;

static FILE			*Logf = NULL;
static char			*log_file = "udp_s.log";
static int			port = 9000;
static short			ipv4 = 1, ipv6 = 1;
static int			maxconn = 0;
static struct sockaddr_in	from;
static socklen_t		from_len;


static void s_send(struct Conn *C)
{
	int fd;
	ssize_t ret;

	Log(0, "%s\n", __FUNCTION__);

	fd = Conn_get_fd(C);
	from_len = sizeof(from);
	ret = sendto(fd, "xxx\n", 4, 0, (struct sockaddr_in *) &from, from_len);
	Log(0, "%s: ret = %d\n", __FUNCTION__, ret);
}

static void s_recv(struct Conn *C)
{
	int fd, len;
	unsigned char buf[10240];
	char *dump;
	ssize_t ret;

	Log(0, "%s...\n", __FUNCTION__);

	fd = Conn_get_fd(C);
	from_len = sizeof(from);
	ret = recvfrom(fd, buf, sizeof(buf), 0, (struct sockaddr *) &from,
		&from_len);
	if (ret == -1) {
		Log(0, "Error %d!\n", ret);
		return;
	}
	len = ret;

	if (debug >= 8) {
		dump = Conn_dump(buf, len);
		Log(8, "%s: recv head=%d tail=%d bytes on slot %d: [%s]\n",
			__FUNCTION__, C->ibuf_head, C->ibuf_tail,
			C->slot, dump);
		free(dump);
	}

	Log(8, "Send back: %s", buf);
	Conn_enqueue(C, buf, len);
}

static void s_error(struct Conn *C)
{
	Log(1, "%s: slot=%d, id=%llu\n",
		__FUNCTION__, C->slot, Conn_getid(C));
}

int main(void)
{
	struct Conn *I4 = NULL, *I6 = NULL;
	int ret;
	struct timeval start;
	int loops = 0;

	Logf = fopen(log_file, "w");
	if (!Logf) {
		fprintf(stderr, "Cannot open log file [%s] [%s]\n",
			log_file, strerror(errno));
		return 1;
	}
	if (debug > 0)
		setlinebuf(Logf);

	/*daemon(0, 0);*/

	gettimeofday(&start, NULL);
	Log(0, "\nStarting at %ld...\n", start.tv_sec);
	Log(0, "\tPort=%d\n", port);
	Log(0, "\tLogFile=%s Debug=%d\n", log_file, debug);
	Log(0, "\tipv4=%s ipv6=%s\n", (ipv4 == 1) ? "on" : "off", (ipv6 == 1) ? "on" : "off");

	/* set library output debug */
	Conn_debug(Logf, debug);

	ret = Conn_init(maxconn);
	if (ret == -1) {
		Log(0, "%s", Conn_strerror());
		return 1;
	}

	if (ipv4 == 1) {
		Log(9, "Try to register IPv4 socket...\n");
		I4 = Conn_socket(PF_INET, SOCK_DGRAM, port);
		if (!I4)
			Log(1, "Cannot bind on ipv4 socket [%s].\n", Conn_strerror());
	}

	if (ipv6 == 1) {
		Log(9, "Try to register IPv6 socket...\n");
		I6 = Conn_socket(PF_INET6, SOCK_DGRAM, port);
		if (!I6)
			Log(1, "Cannot bind on ipv6 socket [%s].\n", Conn_strerror());
	}

	if ((I6 == NULL) && (I4 == NULL)) {
		Log(0, "Cannot bind!\n");
		return 1;
	}

	Conn_recv_cb = s_recv;
	Conn_send_cb = s_send;
	Conn_error_cb = s_error;

	while (1) {
		ret = Conn_poll(10000);
		if (ret == -1) {
			Log(0, "Error calling Conn_poll [%s]!\n", Conn_strerror());
			return 1;
		}

		loops++;
		if (ret == 0)
			break;

		/*
		if (loops > 1000000)
			break;
		*/
	}

	Log(0, "Finish!\n\n");

	return 0;
}
